
/*
   Copyright (c) 1993,1994, UK, John M.Howells

   Program to display the version of Windows on which it is run.
   The program can be compiled for botj 16-bit and 32-bit versions.

   It will report:

	16-bit version:	either Windows 3.1 'proper' or WOW (i.e. Windows on
			 Windows, the 16-bit emulation provided by Windows NT)

   or

	32-bit version:	either Windows NT 'proper' or Chicago or Win32s
			 (the extension to allow 32-bit programs to run on
			 the 16-bit version of Windows 3.1x)

   If running on 16-bit Windows 3.1x the program checks to see if Win32s is
   available, and if so reports the Win32s version and build.

   The source of the information is as follows:

	Detecting Windows-on-Windows:		Knowledgebase Article Q92395
	Detecting Win32s:			Knowledgebase Article Q92395
	Detecting the Version of Win32s:	Win32s Programmer's Reference
	Other Version information:		Knowledgebase Article Q92395

   When the program is running on a 16-bit version of Windows 3.1x or later
   it uses the DOS interface to check the version of Windows. It does this
   because Winfows for Workgroups 3.11 tells lies, and reports that it is
   Windows 3.10.

 */


#include	<windows.h>


#define		PROG_NAME	"Windows Version"


#define		WF_WINNT	0x4000		/* flags for WOW in 16-bit Windows */
#define		W32SYS_DLL	"W32SYS.DLL"	/* name of Win32s DLL */


#if	__BORLANDC__ >= 0x0452
/* on Borland C++ 4.0 exclude exception handling code */
void	_ExceptInit( void )
{
	;
}
#endif


/* the function that drives the whole thing */
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
		    LPSTR lpszCmdLine, int nCmdShow )
{
	DWORD	win_vers;
	char	buff[ 1024 ];

#if	defined( WIN32 ) || defined( _WIN32 ) || defined( __WIN32__ )

	/* program is being compiled for 32-bit */

	win_vers = GetVersion();
	if( ( win_vers & 0x80000000 ) == 0 )
	{
		/* this is real Windows NT */
		wsprintf( buff, "Version %d.%02d Build %d of Windows NT",
				LOBYTE( LOWORD( win_vers ) ),
				HIBYTE( LOWORD( win_vers ) ),
				HIWORD( win_vers ) & 0x7FFF );
		MessageBox( NULL, buff, PROG_NAME " : 32-bit",
				MB_OK | MB_ICONINFORMATION );
		return( FALSE );
	}
	else
	{
		if( LOBYTE( LOWORD( win_vers ) ) < 4 )
		{
			/* this is Win32s running on 16-bit Windows 3.1x,
			   because the program is running in 32-bit mode it
			   cannot easily determine the version of Win32s as
			   that info comes from a 16-bit DLL - good innit.
			   the program *could* work out the version, but it
			   would require a 16-bit DLL to thunk to, and that
			   just is not worth the effort. */
			wsprintf( buff, "      Win32s running on Version %d.%02d\n"
					"\t      of Windows\n"
					"\n"
					"      Run 16-bit version for information\n"
					"                 on Version of Win32s\n"
					"and for accurate 16-bit build information",
					LOBYTE( LOWORD( win_vers ) ),
					HIBYTE( LOWORD( win_vers ) ) );
			MessageBox( NULL, buff, PROG_NAME " : 32-bit",
					MB_OK | MB_ICONINFORMATION );
			return( FALSE );
		}
		else
		{
			/* this is Chicago - Windows 4.x */
			wsprintf( buff, "Chicago Version %d.%02d",
					LOBYTE( LOWORD( win_vers ) ),
					HIBYTE( LOWORD( win_vers ) ) );
			MessageBox( NULL, buff, PROG_NAME " : 32-bit",
					MB_OK | MB_ICONINFORMATION );
			return( FALSE );
		}
	}

	/* end of 32-bit Windows code */

#else

	/* start of 16-bit Windows code */

	DWORD	win_flags;
	extern	WORD	get_dos_win_ver( void );

	win_vers = GetVersion();
	win_flags = GetWinFlags();
	if( win_flags & WF_WINNT )
	{
		/* Program is running on Windows NT using the
		   16-bit Windows on Windows emulation */
		wsprintf( buff, "WOW (Win16) on Version %d.%02d\n"
				"\tof Windows NT",
				LOBYTE( LOWORD( win_vers ) ),
				HIBYTE( LOWORD( win_vers ) ) );
		MessageBox( NULL, buff, PROG_NAME " : 16-bit",
				MB_OK | MB_ICONINFORMATION );
		return( FALSE );
	}
	else
	{
		/* Program is running on real 16-bit Windows 3.1x - here the program
		   can find out which version, if any, of Win32s is available, and
		   which particular build of Windows 3.1x this really is */

		/* data for Win32s */
		typedef	struct
		{
			BYTE	major;
			BYTE	minor;
			WORD	build;
			BOOL	debug;
		} WIN32SINFO, far *LPWIN32SINFO;

		/* data for Win32s */
		HANDLE		h_win32s;       	/* handle to Win32s library */
		FARPROC		getwin32s_info;		/* address of Win32s version function */
		WIN32SINFO	win32s_info;		/* the Win32s version information */
		char		temp[ 1024 ];		/* filled with Win32s version - if any */
		char		bld_id[ 128 ];		/* for true build information */

		if( LOBYTE( LOWORD( win_vers ) ) >= 3 &&
		    HIBYTE( LOWORD( win_vers ) ) >= 10 )
		{
			WORD		true_win_vers;

			/* this bit is because of Windows for Workgroups 3.11 */
			true_win_vers = get_dos_win_ver();
			if( true_win_vers == 0xFFFF ||
			    ( LOBYTE( LOWORD( win_vers ) ) == HIBYTE( true_win_vers ) &&
			      HIBYTE( LOWORD( win_vers ) ) == LOBYTE( true_win_vers ) ) )
			{
				/* either no version information from DOS or it
				   is the same as the Windows version */
				lstrcpy( bld_id, "" );
			}
			else
			{
				/* DOS interface gave us version information */
				wsprintf( bld_id, "     True Windows Version %d.%d\n",
					HIBYTE( true_win_vers ),
					LOBYTE( true_win_vers ) );
			}
		}

		SetErrorMode( SEM_NOOPENFILEERRORBOX );
				/* disable "File Not Found" error box */

		/* now find out what (if any) version of Win32s is available */
		h_win32s = LoadLibrary( W32SYS_DLL );	/* try and load Win32s library */
		if( h_win32s > HINSTANCE_ERROR )
		{
			/* library loaded so get address of info function */
			getwin32s_info = GetProcAddress( h_win32s, "GETWIN32SINFO" );
			if( getwin32s_info )
			{
				/* Win32s Version 1.1 or later installed */
				if( !(*getwin32s_info)((LPWIN32SINFO)&win32s_info))
				{
					wsprintf( temp, " Supporting Version %d.%d Build %d\n"
									"\tof Win32s",
								win32s_info.major,
								win32s_info.minor,
								win32s_info.build );
				}
				else
				{
					wsprintf( temp, "???" );
				}
			}
			else
			{
				/* if no function then Win32s is version 1.0 */
				wsprintf( temp, "Supporting Version 1.0\n"
							"\tof Win32s" );
			}
			FreeLibrary( h_win32s );
		}
		else
		{
			/* no Win32s library */
			wsprintf( temp, "          Win32s not available" );
		}
		wsprintf( buff, "Version %d.%02d of Windows (16-bit)\n"
						"%s"
						"        on Version %d.%02d of DOS\n",
				LOBYTE( LOWORD( win_vers ) ),
				HIBYTE( LOWORD( win_vers ) ),
				(LPSTR)bld_id,
				HIBYTE( HIWORD( win_vers ) ),
				LOBYTE( HIWORD( win_vers ) ) );
		lstrcat( buff, temp );
		MessageBox( NULL, buff, PROG_NAME " : 16-bit",
				MB_OK | MB_ICONINFORMATION );
		return( FALSE );
	}
	
	/* end of 16-bit code */

#endif

}


#if	!defined( WIN32 ) && !defined( _WIN32 ) && !defined( __WIN32__ )

/* support function only used in 16-bit mode */

#define		MPX_INT		0x2f		/* DOS Multiplex Interrupt */
#define		MPX_WIN_VER	0x160A		/* Get Windows Version */
#define		MPX_CHK_MODE	0x1686		/* Check for program mode */

#define		DPMI_INT	0x31		/* DPMI Interrupt */
#define		DPMI_REAL_INT	0x0300		/* DPMI call real-mode int */

/* this uses the DPMI interface to call real-mode INT 0x2F
   to get Windows version as seen by a DOS program. If the
   program directly uses INT 0x2F it gets a protected mode
   interupt that does not support the required interface.

   Thanks to Paul A LeBlanc of Borland's TeamB on CIS's BCPPWIN
   forum for suggesting the use of a DPMI real-mode interrupt
   for getting at the Windows version reported to a DOS program. */
WORD	get_dos_win_ver( void )
{
	/* DPMI Function 0x0300 register set */
	struct	real_regs
	{
		DWORD	reg_edi;
		DWORD	reg_esi;
		DWORD	reg_ebp;
		DWORD	reg_reserved;
		DWORD	reg_ebx;
		DWORD	reg_edx;
		DWORD	reg_ecx;
		DWORD	reg_eax;
		WORD	reg_status;
		WORD	reg_es;
		WORD	reg_ds;
		WORD	reg_fs;
		WORD	reg_gs;
		WORD	reg_ip;
		WORD	reg_cs;
		WORD	reg_sp;
		WORD	reg_ss;
	} real_regs;
	static	struct	real_regs far *real_reg_ptr;

	/* just make sure program is running in protected mode */
	_asm	{
		mov	ax, MPX_CHK_MODE
		int	MPX_INT
		or	ax, ax
		jz	prot_mode		/* AX zero for prot mode */
		}

	/* real or Virtual 86 mode - is this possible under Windows ? */
	return( 0xFFFF );

prot_mode:
	/* program is already running in protected mode,
	   so it is okay to make DPMI (INT 0x31) calls */
	real_reg_ptr = &real_regs;		/* init ptr to register set */

	real_regs.reg_ss = 0;			/* set SS:SP to zero ... */
	real_regs.reg_sp = 0;			/* ... just to make sure */
	real_regs.reg_eax = MPX_WIN_VER;	/* MPX fn to get WIN version */

	_asm	{
		les	di, real_reg_ptr	/* ptr to register set */
		mov	bx, MPX_INT		/* INT 0x2f - flags zero */
		mov	cx, 0			/* 0 words to real-mode stack */
		mov	ax, DPMI_REAL_INT	/* DPMI call real-mode int */
		int	DPMI_INT		/* go to it */
		jc	dpmi_bad		/* carry set on error */
		}

	return( LOWORD( real_regs.reg_ebx ) );	/* return word with WIN vers */

dpmi_bad:
	return( 0xFFFF );		  	/* true version not known */
}

#endif

